'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _defineProperty(obj, key, value) { return Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); }

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactLibUpdate = require('react/lib/update');

var _reactLibUpdate2 = _interopRequireDefault(_reactLibUpdate);

var _objectAssign = require('object-assign');

var _objectAssign2 = _interopRequireDefault(_objectAssign);

var _eventEmitter = require('event-emitter');

var _eventEmitter2 = _interopRequireDefault(_eventEmitter);

var _data = require('./data');

var _data2 = _interopRequireDefault(_data);

var _utils = require('./utils');

var STATES = {
  PENDING: 'PENDING',
  DONE: 'DONE'
};

exports['default'] = function (Component, config) {
  config = (0, _objectAssign2['default'])({}, config);
  var queryParams = (0, _objectAssign2['default'])({}, config.queryParams);
  var options = (0, _objectAssign2['default'])({}, config.options);
  var queries = (0, _objectAssign2['default'])({}, config.queries);
  var mutations = (0, _objectAssign2['default'])({}, config.mutations);
  var displayName = (0, _utils.getContainerName)(Component);
  var queryRequests = (0, _utils.getRequestsFromQueries)(queries);
  var mutationRequests = (0, _utils.getRequestsFromQueries)(mutations);
  var initialRequestNames = queryRequests.map(function (_ref) {
    var name = _ref.name;
    return name;
  });
  var initialState = (0, _objectAssign2['default'])({ $containerState: STATES.PENDING }, (0, _utils.getInitialStateFromRequests)(queryRequests));

  return _react2['default'].createClass({
    displayName: displayName,

    getInitialState: function getInitialState() {
      return initialState;
    },

    componentWillMount: function componentWillMount() {
      this.initialRequestNames = initialRequestNames.slice(0);
      this.ee = (0, _eventEmitter2['default'])();
      this.fetcher = (0, _data2['default'])(options);
      this.unregister = this.fetcher.onReceive(this.onReceive);
    },

    componentDidMount: function componentDidMount() {
      this.fetch(queryRequests);
    },

    componentWillUnmount: function componentWillUnmount() {
      this.ee = undefined;
      this.unregister();
    },

    onReceive: function onReceive(_ref2) {
      var request = _ref2.request;
      var newState = _ref2.data;

      if (this.isMounted && !this.isMounted()) {
        return;
      }
      this.setState(_defineProperty({}, request.name, newState));
      var requestNames = this.initialRequestNames.slice(0);
      var i = requestNames.indexOf(request.name);
      if (i > -1) {
        requestNames.splice(i, 1);
        if (!requestNames.length) {
          this.setState({ $containerState: STATES.DONE });
        }
        this.initialRequestNames = requestNames;
      }
    },

    fetch: function fetch(requests, opt) {
      var _this = this;

      var qp = (0, _objectAssign2['default'])({}, queryParams, this.props);
      requests.forEach(function (request) {
        var params = (0, _utils.setParamValues)(request.params, qp);
        _this.fetcher.fetch({ request: request, params: params, options: opt });
      });
    },

    mutate: function mutate(request, requestData) {
      var qp = (0, _objectAssign2['default'])({}, queryParams, this.props);
      var params = (0, _utils.setParamValues)(request.params, qp);
      this.fetcher.run({ request: request, params: params, data: requestData, options: options });
    },

    setQueryParams: function setQueryParams(newParams) {
      var paramNames = Object.keys(newParams);
      var requestsToRedo = queryRequests.filter(function (request) {
        return paramNames.some(function (name) {
          return request.paramDependencies.indexOf(name) > -1;
        });
      });
      queryParams = (0, _reactLibUpdate2['default'])(queryParams, { $merge: newParams });
      this.fetch(requestsToRedo, { force: true });
    },

    render: function render() {
      var _this2 = this;

      this.ee.emit('render');

      if (this.state.$containerState !== STATES.DONE) {
        return null;
      }

      var mutators = mutationRequests.reduce(function (mutatorProps, mutation) {
        return (0, _objectAssign2['default'])(_defineProperty({}, mutation.name, _this2.mutate.bind(_this2, mutation)), mutatorProps);
      }, {});

      var setQueryParams = this.setQueryParams;

      var props = (0, _objectAssign2['default'])({ setQueryParams: setQueryParams }, this.props, this.state, mutators);

      return _react2['default'].createElement(Component, props);
    }
  });
};

module.exports = exports['default'];