'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj['default'] = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _eventEmitter = require('event-emitter');

var _eventEmitter2 = _interopRequireDefault(_eventEmitter);

var _objectAssign = require('object-assign');

var _objectAssign2 = _interopRequireDefault(_objectAssign);

var _rest = require('./rest');

var _rest2 = _interopRequireDefault(_rest);

var _store = require('./store');

var store = _interopRequireWildcard(_store);

function fetcher(opts) {
  var ee = (0, _eventEmitter2['default'])();

  var listeners = [];

  var getListener = function getListener(filter) {
    for (var i = 0, len = listeners.length; i < len; i++) {
      if (filter(listeners[i])) {
        return listeners[i];
      }
    }
    return null;
  };

  var unregister = store.onChange(function (collection) {
    var l = listeners.filter(function (_ref) {
      var request = _ref.request;
      return request.collection === collection;
    });
    l.forEach(function (_ref2) {
      var request = _ref2.request;
      var params = _ref2.params;

      var data = store.getRequestData(request, params);
      if (data) {
        receive({ request: request, params: params, data: data });
      }
    });
  });

  var receive = function receive(_ref3) {
    var request = _ref3.request;
    var params = _ref3.params;
    var data = _ref3.data;

    ee.emit('receive', { request: request, params: params, data: data });
  };

  var change = function change(_ref4) {
    var request = _ref4.request;
    var params = _ref4.params;
    var data = _ref4.data;

    // receive({request, params, data});
    store.setRequestData(request, params, data);
  };

  var listen = function listen(request, params) {
    var listener = getListener(function (_ref5) {
      var req = _ref5.request;
      return req.name === request.name;
    });
    if (listener) {
      listener.params = params;
      return;
    }
    listeners.push({ request: request, params: params });
  };

  var fetch = function fetch(_ref6) {
    var request = _ref6.request;
    var params = _ref6.params;
    var data = _ref6.data;
    var options = _ref6.options;

    if (request.type === 'READ') {
      listen(request, params);
    }
    if (request.type === 'READ' && params.$id) {
      var existingData = store.getRequestData(request, params);
      if (existingData) {
        return receive({ request: request, params: params, data: existingData });
      }
    } else if (request.type === 'UPDATE' || request.type === 'DELETE') {
      change({ request: request, params: params, data: data });
    }
    _rest2['default'].run({ request: request, params: params, data: data, options: (0, _objectAssign2['default'])({}, opts, options) }).then(function (result) {
      return change({ request: request, params: params, data: result });
    })['catch'](function (err) {
      // FIXME: real error handling
      console.error(err);
    });
  };

  return {
    fetch: fetch,

    run: function run(_ref7) {
      var request = _ref7.request;
      var params = _ref7.params;
      var data = _ref7.data;
      var options = _ref7.options;

      fetch({ request: request, params: params, data: data, options: (0, _objectAssign2['default'])({}, options, { force: true }) });
    },

    onReceive: function onReceive(cb) {
      ee.on('receive', cb);
      return function () {
        unregister();
        ee.off('receive', cb);
      };
    }
  };
}

exports['default'] = fetcher;
module.exports = exports['default'];