'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _defineProperty(obj, key, value) { return Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); }

var _objectAssign = require('object-assign');

var _objectAssign2 = _interopRequireDefault(_objectAssign);

var _xhr = require('./xhr');

var _xhr2 = _interopRequireDefault(_xhr);

var TYPES_TO_METHOD = {
  CREATE: 'POST',
  READ: 'GET',
  UPDATE: 'PUT',
  DELETE: 'DELETE'
};
var BODY_METHODS = ['POST', 'PUT'];

function getHttpRequest(queryRequest) {
  var url = '/' + queryRequest.collection;
  if (queryRequest.params.$id) {
    url += '/<$id>';
  }
  return {
    url: url,
    name: queryRequest.name,
    collection: queryRequest.collection,
    params: queryRequest.params,
    paramDependencies: queryRequest.paramDependencies,
    method: TYPES_TO_METHOD[queryRequest.type]
  };
}

function prepareForQueryString(params) {
  return Object.keys(params).reduce(function (obj, key) {
    return (0, _objectAssign2['default'])(obj, _defineProperty({}, key, JSON.stringify(params[key])));
  }, {});
}

function getHttpRequests(queryRequests) {
  return queryRequests.map(getHttpRequest);
}

function hasBody(request) {
  return BODY_METHODS.indexOf(request.method) > -1;
}

function addLeadingSlash(url) {
  if (url[0] !== '/') {
    return '/' + url;
  }
  return url;
}

function stripTrailingSlash(url) {
  if (url[url.length - 1] === '/') {
    return url.slice(0, -1);
  }
  return url;
}

function run(_ref) {
  var request = _ref.request;
  var params = _ref.params;
  var data = _ref.data;
  var options = _ref.options;

  var httpRequest = getHttpRequest(request);
  var url = params.$id ? httpRequest.url.replace('<$id>', params.$id) : httpRequest.url;
  var baseUrl = options.baseUrl || '';
  var path = stripTrailingSlash(baseUrl) + addLeadingSlash(url);
  var config = {
    method: httpRequest.method,
    path: path,
    headers: {
      'Content-Type': 'application/json'
    }
  };

  params = (0, _objectAssign2['default'])({}, params);

  if (params.$id) {
    delete params.$id;
  }

  config.params = prepareForQueryString(params);

  if (hasBody(httpRequest)) {
    config.entity = data;
  }

  return (0, _xhr2['default'])(config).then(function (res) {
    return res.entity;
  }, function (res) {
    if (res.error) {
      throw res.error;
    } else if (res.status) {
      throw res.status;
    }
    throw res;
  });
}

function onPush() {}

var rest = {
  getHttpRequests: getHttpRequests,
  run: run,
  onPush: onPush
};

exports['default'] = rest;
module.exports = exports['default'];

// noop