'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _defineProperty(obj, key, value) { return Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); }

var _reactLibUpdate = require('react/lib/update');

var _reactLibUpdate2 = _interopRequireDefault(_reactLibUpdate);

var _sift = require('sift');

var _sift2 = _interopRequireDefault(_sift);

var _eventEmitter = require('event-emitter');

var _eventEmitter2 = _interopRequireDefault(_eventEmitter);

_sift2['default'].useOperator('id', function (a, b) {
  return a === b.id;
});
_sift2['default'].useOperator('limit', function () {
  return true;
});
_sift2['default'].useOperator('skip', function () {
  return true;
});

var ee = (0, _eventEmitter2['default'])();

var storage = {};

function setState(state) {
  storage = (0, _reactLibUpdate2['default'])(storage, { $merge: state });
  Object.keys(state).forEach(function (key) {
    return ee.emit('change', key);
  });
}

function getRequestData(request, params) {
  var collection = storage[request.collection] || [];
  var data = (0, _sift2['default'])(params, collection);
  if (params.$id) {
    return data && data[0];
  }
  if (params.$skip || params.$limit) {
    var skip = params.$skip || 0;
    var limit = params.$limit ? skip + params.$limit : null;
    data = data.slice(skip, limit);
  }
  return data;
}

function setRequestData(request, params, data) {
  var collection = storage[request.collection] || [];
  var newState = undefined;
  if (params.$id) {
    var i = _sift2['default'].indexOf(params, collection);
    if (i > -1) {
      newState = (0, _reactLibUpdate2['default'])(collection, { $splice: [request.type === 'DELETE' ? [i, 1] : [i, 1, data]] });
    } else if (request.type !== 'DELETE') {
      newState = (0, _reactLibUpdate2['default'])(collection, { $push: [data] });
    }
  } else if (Array.isArray(data)) {
    newState = (0, _reactLibUpdate2['default'])(collection, { $push: data.filter(function (item) {
        return _sift2['default'].indexOf({ $id: item.id }, collection) < 0;
      }) });
  } else {
    newState = (0, _reactLibUpdate2['default'])(collection, { $push: [data] });
  }
  if (newState) {
    setState(_defineProperty({}, request.collection, newState));
  }
  return data;
}

function onChange(listener) {
  ee.on('change', listener);
  return function unregister() {
    ee.off('change', listener);
  };
}

function bootstrap(data) {
  storage = JSON.parse(data);
}

function dump() {
  return JSON.stringify(storage);
}

exports.bootstrap = bootstrap;
exports.dump = dump;
exports.setState = setState;
exports.getRequestData = getRequestData;
exports.setRequestData = setRequestData;
exports.onChange = onChange;