'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
var DEFAULT_TYPE = 'READ';
var PROP_TO_TYPE = {
  $create: 'CREATE',
  $update: 'UPDATE',
  $delete: 'DELETE'
};

function getContainerName(Component) {
  return '' + (Component.displayName || Component.name) + 'Container';
}

function getTypeAndParams(declarationParams) {
  var params = declarationParams || {};
  var type = DEFAULT_TYPE;

  for (var p in PROP_TO_TYPE) {
    if (params[p]) {
      params = params[p];
      type = PROP_TO_TYPE[p];
      break;
    }
  }

  return { type: type, params: params };
}

function isQueryParam(val) {
  return typeof val === 'string' && val[0] === '<' && val[val.length - 1] === '>';
}

function getQueryParamDependencies(params) {
  return Object.keys(params).reduce(function (dependencies, key) {
    if (isQueryParam(params[key])) {
      return dependencies.concat(params[key].slice(1, -1));
    }
    return dependencies;
  }, []);
}

function getInitialStateFromRequest(_ref) {
  var params = _ref.params;

  return params.$id ? {} : [];
}

function getRequestFromQuery(name, query) {
  var collections = Object.keys(query);

  if (collections.length === 0) {
    throw new Error('Missing collection name in query with name: "' + name + '"!');
  } else if (collections.length > 1) {
    throw new Error('Too many collection names in query with name: "' + name + '"! There can be only one collection per query!');
  }

  var collection = collections[0];

  var _getTypeAndParams = getTypeAndParams(query[collection]);

  var type = _getTypeAndParams.type;
  var params = _getTypeAndParams.params;

  var paramDependencies = getQueryParamDependencies(params);

  return { name: name, collection: collection, type: type, params: params, paramDependencies: paramDependencies };
}

function getRequestsFromQueries(queries) {
  var propertyNames = Object.keys(queries);

  return propertyNames.map(function (property) {
    var query = queries[property];
    return getRequestFromQuery(property, query);
  });
}

function getInitialStateFromRequests(requests) {
  return requests.reduce(function (state, request) {
    state[request.name] = getInitialStateFromRequest(request);
    return state;
  }, {});
}

function setParamValues(declarationParams, queryParams) {
  return Object.keys(declarationParams).reduce(function (params, key) {
    if (isQueryParam(declarationParams[key])) {
      params[key] = queryParams[declarationParams[key].slice(1, -1)];
    } else if (declarationParams[key] !== null && typeof declarationParams[key] === 'object') {
      params[key] = setParamValues(declarationParams[key], queryParams);
    } else {
      params[key] = declarationParams[key];
    }
    return params;
  }, {});
}

exports.getContainerName = getContainerName;
exports.getRequestsFromQueries = getRequestsFromQueries;
exports.getInitialStateFromRequests = getInitialStateFromRequests;
exports.setParamValues = setParamValues;